local AddonName, Addon = ...
Addon.Frame = CreateFrame("Frame")
local Frame = Addon.Frame
Frame:Hide() Addon.SetWindow = {}
local L = Addon.L
Addon.Version = GetAddOnMetadata(AddonName, "Version")
local Version = Addon.Version
local IsInGuild = IsInGuild
local IsInRaid = IsInRaid
local IsInGroup = IsInGroup
local IsGuildMember = IsGuildMember
local IsFriend = C_FriendList.IsFriend
local UnitIsGroupLeader = UnitIsGroupLeader
local UnitIsGroupAssistant = UnitIsGroupAssistant
local UnitName = UnitName
local UnitGUID = UnitGUID
local UnitIsDead = UnitIsDead
local UnitInRaid = UnitInRaid
local GetLootInfo = GetLootInfo
local GetLootSlotLink = GetLootSlotLink
local CloseTrade = CloseTrade
local SetCVar = SetCVar
local GetCVar = GetCVar
local print = print
local SendChatMessage = SendChatMessage
local SendAddonMessage = C_ChatInfo.SendAddonMessage
local IsAddonMessagePrefixRegistered = C_ChatInfo.IsAddonMessagePrefixRegistered
local RegisterAddonMessagePrefix = C_ChatInfo.RegisterAddonMessagePrefix
local PrefixLM = "LMPX"
local Broadcast = 41
local HasLooter = {}
local HasItems = 0
local GroupMember = {}
function Addon:GetShortName(SourceName)
    if SourceName then
        return SourceName:match("^(%S*)%-") or SourceName
    else
        return nil
    end
end
function Addon:UpdateTable(Target, Source)
    for k, v in pairs(Source) do
        if type(v) == "table" then
            if type(Target[k]) == "table" then
                self:UpdateTable(Target[k], v)
            else
                Target[k] = self:UpdateTable({}, v)
            end
        elseif type(Target[k]) ~= "table" then
            Target[k] = v
        end
    end return Target
end
function Addon:DoCheck()
    if IsInRaid() and (UnitIsGroupLeader("player") or UnitIsGroupAssistant("player")) then
        SendAddonMessage(PrefixLM, "check "..Version, "raid")
    end
end
function Addon:GetCheckText()
    local i = 1
    local msg1, msg2 = "", ""
    for k, v in pairs(GroupMember) do
        if i < 20 then
            if GroupMember[k] == 0 then
                msg1 = msg1 .. k .. "(" .. L["NoAddon"] .. ")\n"
            else
                msg1 = msg1 .. k .. "(v" .. v .. ")\n"
            end
        elseif i == 20 then
            if GroupMember[k] == 0 then
                msg1 = msg1 .. k .. "(" .. L["NoAddon"] .. ")\n"
            else
                msg1 = msg1 .. k .. "(v" .. v .. ")"
            end
        elseif i > 20 and i < 40 then
            if GroupMember[k] == 0 then
                msg2 = msg2 .. k .. "(" .. L["NoAddon"] .. ")\n"
            else
                msg2 = msg2 .. k .. "(v" .. v .. ")\n"
            end
        elseif i == 40 then
            if GroupMember[k] == 0 then
                msg2 = msg2 .. k .. "(" .. L["NoAddon"] .. ")\n"
            else
                msg2 = msg2 .. k .. "(v" .. v .. ")"
            end
        end i = i + 1
    end return msg1, msg2
end
function Addon:UpdateGroupInfo()
    table.wipe(GroupMember)
    if IsInRaid() then
        for i = 1, 40 do
            if UnitName("raid"..i) and not GroupMember[UnitName("raid"..i)] then
                GroupMember[UnitName("raid"..i)] = 0
            end
        end
    end
end
function Addon:AllHaveLootMonitor()
    for k, v in pairs(GroupMember) do
        if v == 0 then
            return false
        end
    end return true
end

Addon.Config = { ["PreventTrade"] = true, ["OutputToRaidWarning"] = false, ["SpellWhisper"] = false, ["Quality"] = 4,}
local Config = Addon.Config

Frame:RegisterEvent("ADDON_LOADED")
Frame:RegisterEvent("PLAYER_LOGOUT")
Frame:RegisterEvent("MAIL_SHOW")
Frame:RegisterEvent("MAIL_CLOSED")
Frame:RegisterEvent("TRADE_SHOW")
Frame:RegisterEvent("LOOT_OPENED")
Frame:RegisterEvent("CHAT_MSG_ADDON")
Frame:RegisterEvent("ZONE_CHANGED")
Frame:RegisterEvent("GROUP_ROSTER_UPDATE")

Frame:SetScript("OnEvent", function(self, event, ...)
    if type(self[event]) == "function" then
        return self[event](self, ...)
    end
end)
function Frame:ADDON_LOADED(Name)
    if Name ~= AddonName then
        return
    end self:UnregisterEvent("ADDON_LOADED")
    Frame:Hide()
    if not LootMonitorDB then
        LootMonitorDB = {}
    end
    if not LootMonitorDB.Config then
        LootMonitorDB.Config = {}
    end
    Addon:UpdateTable(Config, LootMonitorDB.Config)
    if not IsAddonMessagePrefixRegistered(PrefixLM) then
        RegisterAddonMessagePrefix(PrefixLM)
    end
    if not Config.SpellWhisper and GetCVar("portal") == "CN" then
        ConsoleExec("portal US")
        ConsoleExec("profanityFilter 0")
        ConsoleExec("overrideArchive 0")
    else
        self:UnregisterEvent("TRADE_SHOW") self:UnregisterEvent("MAIL_CLOSED") self:UnregisterEvent("MAIL_SHOW")
    end
    Addon.SetWindow:Initialize()
end
function Frame:PLAYER_LOGOUT()
    if IsAddOnLoaded("SpellWhisper") then
        Config.SpellWhisper = true
        Config.PreventTrade = false
    else
        Config.SpellWhisper = false
    end
    LootMonitorDB = { ["Config"] = {}, }
    Addon:UpdateTable(LootMonitorDB.Config, Config)
end
function Frame:TRADE_SHOW()
    if not Config.PreventTrade then
        return
    end
    local RequestName = UnitName("npc")
    local TargetCanTradeMe = false
    if UnitName("target") == RequestName then
        TargetCanTradeMe = true
    end
    if UnitGUID("npc") and not TargetCanTradeMe then
        if IsFriend(UnitGUID("npc")) then
            TargetCanTradeMe = true
        end
    end
    if IsInGuild() and not TargetCanTradeMe then
        if IsGuildMember(RequestName) then
            TargetCanTradeMe = true
        end
    end
    if IsInRaid() and not TargetCanTradeMe then
        for i = 1, 40 do
            local u = "raid"..i
            if UnitName(u) and UnitName(u) == RequestName then
                TargetCanTradeMe = true break
            end
        end
    elseif IsInGroup() and not TargetCanTradeMe then
        for i = 1, 4 do
            local u = "party"..i
            if UnitName(u) and UnitName(u) == RequestName then
                TargetCanTradeMe = true break
            end
        end
    end
    if not TargetCanTradeMe then
        CloseTrade()
    end
end
function Frame:MAIL_CLOSED()
    SetCVar("blockTrades", 0)
end
function Frame:MAIL_SHOW()
    SetCVar("blockTrades", 1)
end
function Frame:ZONE_CHANGED()
    table.wipe(HasLooter)
end
function Frame:CHAT_MSG_ADDON(...)
    if not IsInRaid() then
        return
    end
    local arg = {...}
    if arg[1] == PrefixLM then
        local CMD, REST = arg[2]:match("(%S+)%s+(.-)$")
        local OutputChannel = "raid"
        if Config.OutputToRaidWarning then
            OutputChannel = "raid_warning"
        end
        if CMD:lower() == "start" then
            GroupMember[arg[5]] = tonumber(REST)
            if UnitInRaid(arg[5]) < Broadcast then
                Broadcast = UnitInRaid(arg[5])
            end
        elseif CMD:lower() == "check" then
            SendAddonMessage(PrefixLM, "reply "..tostring(Version), "raid")
            if tonumber(REST) > tonumber(Version) then
                print(L["<LM>Need Update your LootMonitor Addon to newest one!"])
            end
        elseif CMD:lower() == "reply" then
            GroupMember[arg[5]] = tonumber(REST)
            if UnitInRaid(arg[5]) < Broadcast then
                Broadcast = UnitInRaid(arg[5])
            end
        elseif CMD:lower() == "target" then
            local TargetName, TargetGUID = REST:match("(%S+)%s(.-)$")
            if not HasLooter[TargetGUID] then
                HasLooter[TargetGUID] = arg[5]
                HasItems = 1
                if Broadcast == UnitInRaid("player") and Addon:AllHaveLootMonitor() then
                    SendChatMessage(string.format(L["<LM><%s> has Looted %s's Corpse"], arg[5], TargetName), OutputChannel)
                end
            end
        elseif CMD:lower() == "loot" and HasItems ~= 0 then
            local CorpseGUID, LootLink = REST:match("(%S+)%s+(.-)$")
            if HasLooter[CorpseGUID] == arg[5] then
                if Broadcast == UnitInRaid("player") then
                    SendChatMessage(string.format("%d: %s", HasItems, LootLink), OutputChannel)
                end
                HasItems = HasItems + 1
            end
        elseif CMD:lower() == "end" and HasItems ~= 0 then
            local CorpseGUID, LootNum = REST:match("(%S+)%s+(%d+)$")
            if HasLooter[CorpseGUID] == arg[5] then
                if Broadcast == UnitInRaid("player") then
                    SendChatMessage(string.format(L["Total %s Items"], LootNum), OutputChannel)
                end
                HasItems = 0
            end
        end
    end
end
function Frame:GROUP_ROSTER_UPDATE()
    Addon:UpdateGroupInfo()
    if IsInRaid() and not UnitInBattleground("player") then
        SendAddonMessage(PrefixLM, "start "..tostring(Version), "raid")
    end
end
function Frame:LOOT_OPENED()
    if not IsInGroup() then
        return
    end
    if not UnitIsDead("target") then
        return
    end
    local loots = GetLootInfo()
    local msg = {}
    for i = 1, #loots do
        if loots[i] then
            if loots[i].quantity ~= 0 then
                if loots[i].quality >= Config.Quality then
                    table.insert(msg, GetLootSlotLink(i))
                end
            end
        end
    end
    if #msg > 0 then
        SendAddonMessage(PrefixLM, "target " .. UnitName("target") .. " " .. UnitGUID("target"), "raid")
        for i = 1, #msg do
            SendAddonMessage(PrefixLM, "loot ".. UnitGUID("target") .. " " .. msg[i], "raid")
        end
        SendAddonMessage(PrefixLM, "end " .. UnitGUID("target") .. " " .. tostring(#msg), "raid")
    end
end
SLASH_LMC1 = "/lootmonitor"
SLASH_LMC2 = "/lm"
SlashCmdList["LMC"] = function(Input)
    if Addon.SetWindow:IsShown() then
        Addon.SetWindow:Hide()
    else
        Addon.SetWindow:Show()
    end
end